
 ifndef __STRING_SERVICE

#define  __STRING_SERVICE  1  ; para outros servicos reconhecerem a
                         ; presenca deste servico




          variable _SS_PROCESSOR_

_SS_PROCESSOR_ SET 0

          IFDEF  __18F2550
_SS_PROCESSOR_ SET    18
          ENDIF

          IFDEF  __18F4520
_SS_PROCESSOR_ SET    18
          ENDIF

          IFDEF  __18F4550
_SS_PROCESSOR_ SET    18
          ENDIF

          IFDEF  __18F4620
_SS_PROCESSOR_ SET    18
          ENDIF

          IF _SS_PROCESSOR_ == 18

; -----------------------------------------------------------------------------------------------
; STRING_SERVICES                                                                      19/11/2014
;
; STATUS: FUNCIONA
;
; Funcoes de tratamento de strings para a familia de microcontroladores PIC18F.
;
; -----------------------------------------------------------------------------------------------
;
; PRE-REQUISITOS:
;
; 1 - Definir os registradores:
;str_dlm       RES     1
;str_reg1      RES     1
;str_reg2      RES     1
;str_reg3      RES     1
;str_reg4      RES     1
;str_reg5      RES     1
;str_reg6      RES     1
;str_ptr       RES     1
;str_buffer    EQU     0x???    ; buffer auxiliar (colocar endereco 0x??? na RAM)
;
; 2 - As funcoes aqui apresentadas usam strings na memoria RAM com o formato:
;
;     +----+----+----+----+-s   s---+----+----+
;     | LL | cc | cc | cc |         | cc | DL |
;     +----+----+----+----+-s   s---+----+----+
;     onde:
;     LL ..... e' o tamanho da string. 
;     cc ..... sao os caracteres da string.
;     DL .....   o carater delimitador definido previamente pela 
;              funcao 'str_setdlm'.
;
;
; -----------------------------------------------------------------------------------------------


;
; -----------------------------------------------------------------------------------------------
; As funcoes devem ser chamadas atraves das macros definidas abaixo
; -----------------------------------------------------------------------------------------------
;
; CLC addr1,  addr2.......................... Compara duas strings na memoria RAM 
;
; CLI addr, str.............................. Compara uma string na memoria RAM com uma string inline na macro
;
; COPYSTR addr, str.......................... Move a string inline na macro para um local na memoria RAM
;
; DC addr, str............................... OBSOLETA - Define uma string na memoria RAM.
;                                             PREFIRA A MACRO 'COPYSTR'
;
; DLMSET dlm................................. Especifica o delimitador de strings
;
; FIND  addr, char........................... Procura o caracter '_char_' dentro da string '_addr_'
;                                             que se encontra na memoria RAM.
;
; LDW  str................................... Carrega W com um caracter de uma string na RAM 
;
; MOV_STR st................................. OBSOLETA - Copia a string inline na macro para o buffer apontado
;                                             por FSR0 --> PREFIRA A MACRO 'COPYSTR'
;
; MVC origem, len, destino, opt.............. Move localizacao da RAM de 'origem' com tamanho 'len' 
;                                             para a localizacao da RAM 'destino'.
;
; STRINGFY origem, destino................... Copia uma string colocando o tamanho e o delimitador 
;                                             no destino.
;
; SUBSTR  str_addr, substr_addr.............. Procura uma substring dentro de uma string
;
; SUBSTRI str_addr, substr................... Procura a substring 'substr', inline na macro, dentro 
;                                             da string 'str_addr'
;
; -----------------------------------------------------------------------------------------------

STACK     equ    1            ; macro MVC salva FRS0 e FSR1 no stack de software                   
NOSTACK   equ    2            ; macro MVC nao salva FRS0 e FSR1 no stack de software       
; ---------------------------------------------------------------------------------------------
; MVC                                                                                21/05/2015
;
; Move localizacao da RAM de 'origem' com tamanho 'len' para a localizacao
; da RAM 'destino'.
;
; Se desejar que a macro MVC salve os registradores FSR0 e FSR1 no stack de software, use STAK
; na opcao 'opt', senao use 'NOSTACK'. Note que sera preciso incluir no programa o arquivo
; 'stck_services.inc' e seguir as orientacoes nele contidas para usar seus servicos.
;
; ---------------------------------------------------------------------------------------------
MVC        MACRO   origem, len, destino, opt
   ifndef _numq
         variable _numq
_numq    set   0
   endif
_numq    set   _numq + 1

 if opt==STACK
    movf  FSR0L,w
    call  pushW
    movf  FSR0H,w
    call  pushW
    movf  FSR1L,w
    call  pushW
    movf  FSR1H,w
    call  pushW
 endif
           lfsr    FSR0, origem
           lfsr    FSR1, destino
           movlw   len
           movwf   str_reg1
           call    _simple_mvc

 if opt==STACK
    call  popW
    movwf FSR1H
    call  popW
    movwf FSR1L
    call  popW
    movwf FSR0H
    call  popW
    movwf FSR0L
  endif


           ENDM

; ---------------------------------------------------------------------------------------------
; LDW - Carrega W com um caracter de uma string na RAM                               23/07/2015
;
; A string na RAM deve ter o formato recomendado.
;
; Esta funcao nao considera o delimitador da string.
;
; Inicialmente a string sera apontada pelo registrador FSR0 e carregada em W.
; Apos a carga, o registrador ficara apontando para a proxima posicao na string.
; Se o registrador FSR0 for usado para outro proposito apos esta macro, o apontador 
; sera perdido.
; ---------------------------------------------------------------------------------------------
LDW            MACRO   str
               lfsr    FSR0, str+.1              ; pula o tamanho
               movf    POSTINC0,w
               ENDM

; ---------------------------------------------------------------------------------------------
; STRINGFY                                                                         23/04/2015
;
; Copia uma string colocando o tamanho e o delimitador no destino.
;
; A string de origem deve ser delimitada com o delimitador especificado em 'srt_dlm".
;
;  String origem  -->  xxxxxxxxxxxxxxD
;  String destino -->  LxxxxxxxxxxxxxxD
; ---------------------------------------------------------------------------------------------
STRINGFY   MACRO  origem, destino
           lfsr  FSR0, origem
           lfsr  FSR1, destino
           call  cvstring
           ENDM

; -----------------------------------------------------------------------------------------------
; FIND  _addr_, _char_
;
; Procura o caracter '_char_' dentro da string '_addr_' que se encontra na memoria RAM.
;
; A string na RAM deve ter o formato recomendado.
;
; Retorno: STATUS<Z>=0 ........ nao achou
;          STATUS<Z>=1 ........ achou. Neste caso, INDF0 aponta o caracter encontrado
;
; Exemplo:
;RAM400       UDATA   0x400
;cor          RES     .16
;
;             COPYSTR cor, "azul"
;             FIND    cor, "z"
;             bz      achou
;naoachou     retlw   0xFF
;achou        retlw   0x00
;
; -----------------------------------------------------------------------------------------------
FIND     MACRO  addr, char
         lfsr   FSR0, addr
         movlw  char
         call   str_schar
         ENDM

; -----------------------------------------------------------------------------------------------
; SUBSTR  str_addr, substr_addr
;
; Procura uma substring dentro de uma string onde:
;
; str_addr ............... Endereco da string na RAM.
;                          Esta string deve ter o formato recomendado.
; str_substr ............. Endereco da substring na RAM.
;                          Esta string deve ter o formato recomendado.
;
; Apos a procura, os flags do registrador STATUS serao posicionados: 
; str_ptr ........... posicao da substring dentro da string                   
; Z=0 ............... substring nao encontrada.
; Z=1 ............... substring encontrada 
;                     W contem a posicao relativa da substring dentro da string.
;                     FSR0 aponta para o ultimo caracter da substring encontrada
;                     dentro da string.
; N=1 ............... a substring e' maior do que a string.
;
; EXEMPLO:
;RAM400      UDATA    0x400
;nome1       res      .32
;nome2       res      .32
;
;            DLMSET   0x0D
;            COPYSTR  nome1,"Jose Antonio Feliz"
;            COPYSTR  nome2,"Feliz"
;            SUBSTR   nome1, nome2
;            bz       achou
;
; -----------------------------------------------------------------------------------------------
SUBSTR         MACRO    str_addr, substr_addr

  ifndef __dlmsw
    error "Delimitador de strings nao definido pela macro DLMSET"
  endif

   ifndef _numq
         variable _numq
_numq    set   0
   endif
_numq    set   _numq + 1
               lfsr    FSR0,str_addr             ; endereco da string
               lfsr    FSR1,substr_addr          ; endereco da substring
               movf    INDF1,w                   ; tamanho da substring
               subwf   INDF0,w                   ; if (string < substring) {
               bn      _substr#v(_numq)          ;     goto ... }
               lfsr    FSR0,str_addr+1           ; endereco do primeiro caracter na string
               lfsr    FSR1,substr_addr+1        ; endereco do primeiro caracter na substring
               call    str_substr                ; vai procurar a substring
_substr#v(_numq) 
               ENDM


; -----------------------------------------------------------------------------------------------
; SUBSTRI  str_addr, substr                                                            23/04/2015
;
; Procura a substring 'substr', inline na macro, dentro da string 'str_addr' onde:
;
; str_addr ............... Endereco da string na RAM.
;                          A string na RAM deve ter o formato recomendado
; substr ................. Substring inline entre aspas duplas.
;
; Apos a procura, os flags do registrador STATUS serao posicionados:        
; str_ptr ........... posicao da substring dentro da string           
; Z=0 ............... substring nao encontrada.
; Z=1 ............... substring encontrada 
;                     W contem a posicao relativa da substring dentro da string.
;                     FSR0 aponta para o ultimo caracter da substring encontrada
;                     dentro da string.
; N=1 ............... a substring e' maior do que a string.
;
; EXEMPLO:
;RAM400      UDATA    0x400
;nome1       res      .32
;
;            DLMSET   0x0D
;            COPYSTR  nome1,"Jose Antonio Feliz"
;            SUBSTRI  nome1, "Feliz"
;            bz       achou
;
;
; -----------------------------------------------------------------------------------------------
SUBSTRI        MACRO   str_addr, substr  

  ifndef __dlmsw
    error "Delimitador de strings nao definido pela macro DLMSET"
  endif
        
   ifndef _numq
         variable _numq
_numq    set   0
   endif
_numq    set   _numq + 1
               lfsr    FSR1, str_buffer+1        ; inicio dos caracteres no buffer auxiliar
               call    _str_fmt                  ; vai formatar a string 2 (inline na macro)
               DW      substr,0x00               ; esta e' a string 2

               lfsr    FSR0,str_addr             ; endereco da string
               lfsr    FSR1,str_buffer           ; endereco da substring
               movf    INDF1,w                   ; tamanho da substring
               subwf   INDF0,w                   ; if (string < substring) {
               bn      _substi#v(_numq)          ;     goto ... }
               lfsr    FSR0,str_addr+1           ; endereco do primeiro caracter na string
               lfsr    FSR1,str_buffer+1         ; endereco do primeiro caracter na substring
               call    str_substr                ; vai procurar a substring
_substi#v(_numq) 
               ENDM


; -----------------------------------------------------------------------------------------------
; CLC     _addr1_, _addr2_
;
; Compara duas strings na memoria RAM onde:
;
; _addr1_ ................ endereco da string 1.
; _addr2_ ................ endereco da string 2.
;
; As strings na RAM devem ter o formato recomendado 
;
; Apos a comparacao, os flags do registrador STATUS serao posicionados:                   
; Z=0 ............... as strings sao diferentes
; Z=1 ............... as strings sao iguais
;
; -----------------------------------------------------------------------------------------------
; EXEMPLO
;
;RAM400    UDATA   0x400
;nome1     res     .32
;nome2     res     .32
;
;          DLMSET   0x0D
;          COPYSTR  nome1,"abcdefghi"
;          COPYSTR  nome2,"ABCDEFGHI"
;          CLC  nome1, nome2
;          BE   iguais
;          BNE  diferentes
;
;iguais     nop
;diferentes nop
;
; -----------------------------------------------------------------------------------------------
CLC         MACRO    addr1,  addr2

  ifndef __dlmsw
    error "Delimitador de strings nao definido pela macro DLMSET"
  endif

            lfsr     FSR1,addr1
            lfsr     FSR2,addr2
            call     str_cp
            ENDM

; -----------------------------------------------------------------------------------------------
; CLI   _addr_, "cccc......cc"
;
; Compara uma string na memoria RAM com uma string inline na macro onde: 
;
;  _addr_ .............. Endereco da string na memoria RAM.
;  "cccc......cc" ...... String inline na macro.
;
;  A string na RAM deve ter o seguinte formato:  LLcccc......ccDL, onde:]
;
;            +----+----+----+----+-s   s---+----+----+
;            | LL | cc | cc | cc |         | cc | DL |
;            +----+----+----+----+-s   s---+----+----+
;
; onde:      LL ..... e' o tamanho da string. 
;            cc ..... sao os caracteres da string.
;            DL .....   o carater delimitador definido previamente pela 
;                     funcao 'str_setdlm'.
;
; Apos a comparacao, os flags do registrador STATUS serao posicionados:                   
; Z=0 ............... as strings sao diferentes
; Z=1 ............... as strings sao iguais
;
; -----------------------------------------------------------------------------------------------
; EXEMPLO:
;
;RAM400     UDATA   0x400
;nome       res     .32
;endereco   res     .32
;telefone   res     .16
;
;           DLMSET   0x0D
;           COPYSTR  nome,"Pafuncio Silva"
;           COPYSTR  endereco,"RUA Mileva,16"
;           COPYSTR  telefone,"(11)24354-9845"
;
;           CLI nome,"Pafuncio Silva"
;           BE  iguais
;           BNE diferentes
;
;
;iguais         goto inicio
;diferentes     goto inicio
;
; -----------------------------------------------------------------------------------------------
CLI      MACRO   addr, str

  ifndef __dlmsw
    error "Delimitador de strings nao definido pela macro DLMSET"
  endif

   ifndef _numq
         variable _numq
         variable _axbuff
_numq    set   0
_axbuff  set   0
   endif
_numq    set   _numq + 1
               lfsr    FSR1, str_buffer+1        ; inicio dos caracteres no buffer auxiliar
               call    _str_fmt                  ; vai formatar a string 2 (inline na macro)
               DW      str,0x00                  ; esta e' a string 2
               bnz     _cp1#v(_numq)             ; RC != 0 --> retorna com erro (estouro)
               lfsr    FSR1,addr                 ; aponta para a str 1
               lfsr    FSR2,str_buffer           ; aponta para a str 2
               call    str_cp                    ; vai comparar
_cp1#v(_numq)
               ENDM

;
; -----------------------------------------------------------------------------------------------
; DLMSET dml                                                                           23/04/2015
;
; Especifica o delimitador de strings que sera usado pelos servicos deste modulo.
; O delimitador sera colocado no registrador 'str_dlm".
;
; -----------------------------------------------------------------------------------------------
DLMSET   MACRO   dlm
         movlw   dlm
         call    str_setdlm
  ifndef __dlmsw
#define  __dlmsw
  endif

         ENDM

; -----------------------------------------------------------------------------------------------
; COPYSTR  _addr_, "cccc......cc"    (o mesmo que DC)
;
; Move a string inline na macro para um local na memoria RAM onde:
;
; addr_ .............. endereco da RAM onde a string sera criada.
; "cccc....cc" ....... string inline na macro.
;
; A localizacao de memoria _addr_, recebera a string inline na macro e sera configurada
; com o formato 'LLcccc......ccDL', onde:
; LL ........ e' o tamanho da string 
; 'ccc...cc'  e' o conteudo da string
; DL ........   o delimitador definido previamente pela 
;             funcao 'str_setdlm'.
;
; Retorno: STATUS<Z>=1 ........ string criada com sucesso
;          STATUS<Z>=0 ........ string excedeu o tamanho limite
;
; -----------------------------------------------------------------------------------------------
; EXEMPLO:
;
;RAM400     UDATA   0x400
;nome       res     .32
;endereco   res     .32
;telefone   res     .16
;
;           DLMSET   0x0D
;           COPYSTR  nome,"Pafuncio Silva"
;           COPYSTR  endereco,"RUA Mileva,16"
;           COPYSTR  telefone,"(11)24354-9845"
; -----------------------------------------------------------------------------------------------

; -----------------------------------------------------------------------------------------------
COPYSTR  MACRO   addr, str

  ifndef __dlmsw
    error "Delimitador de strings nao definido pela macro DLMSET"
  endif

         lfsr    FSR0, addr                ; endereco do tamanho na RAM
         lfsr    FSR1, (addr + 1)          ; endereco dos caracteres na RAM
         call    str_define
         DW      str,0x00
         ENDM

; ---------------------------------------------------------------------------------------------
; MOV_STR -  Copia a string inline na macro para o buffer apontado por FSR0          27/11/2014
;
; PREFIRA A MACRO 'COPYSTR'
;
; Exemplo:     MOV_STR  "Isto e uma mensagem" 
; 
; ---------------------------------------------------------------------------------------------
MOV_STR       MACRO   st 

  ifndef __dlmsw
    error "Delimitador de strings nao definido pela macro DLMSET"
  endif

              call    mov_str
              dw      st,0x00
              ENDM

; -----------------------------------------------------------------------------------------------
; DC   _addr_, "cccc......cc" (o mesmo que COPYSTR)
;
; Define uma string na memoria RAM onde:
;
; PREFIRA A MACRO 'COPYSTR'
;
; -----------------------------------------------------------------------------------------------
DC       MACRO   addr, str

  ifndef __dlmsw
    error "Delimitador de strings nao definido pela macro DLMSET"
  endif

         lfsr    FSR0, addr                ; endereco do tamanho na RAM
         lfsr    FSR1, (addr + 1)          ; endereco dos caracteres na RAM
         call    str_define
         DW      str,0x00
         ENDM


; -----------------------------------------------------------------------------------------------
; FUNCOES
; -----------------------------------------------------------------------------------------------
; srt_define ............... Define uma string na memoria RAM.
;
; str_schar ................ Procura um caracter dentro da string.
;
; str_setdlm ............... Especifica o delimitador de fim da string.
;
; str_cp ................... Compara caracteres de duas strings.
;
; str_substr ............... Procura uma substring dentro de uma string
;
; _str_fmt ................. Formata uma string na memoria RAM.
;                            Funcao interna usada pela macro CLI.
;
; -----------------------------------------------------------------------------------------------




; -----------------------------------------------------------------------------------------------
; str_substring - Procura uma substring dentro de uma string                           21/11/2014
;
; Entrada ........... FSR0 aponta o endereco de inicio da busca dentro da string.
;                     FSR1 aponta o endereco do primeiro caracter da substring
;
; Saida ............. Apos a procura, os flags do registrador STATUS serao posicionados:                   
;                     Z=0 ............... substring nao encontrada.
;                     Z=1 ............... substring encontrada 
;                                         W contem a posicao relativa da substring dentro da string.
;                                         FSR0 aponta para o ultimo caracter da substring encontrada
;                                         dentro da string.
; ----------------------------------------------------------------------------------------------
str_substr     movf     FSR1H,w                  ; endereco High da substring no stack
               movwf    str_reg3                 ; salva
               movf     FSR1L,w                  ; endereco Low  da substring no stack
               movwf    str_reg4                 ; salva

               movf     FSR0H,w
               movwf    str_reg5                  ; salva
               movf     FSR0L,w
               movwf    str_reg6                  ; salva
str_substr0    movf     INDF1,w                  ; pega o primeiro caracter da substring
               call     str_schar                ; procura esse caractaer dentro da string
               bz       str_substr1              ; achou --> continua
               bcf      STATUS,Z                 ; nao achou, sai da funcao
               return                            ; fim 
str_substr1     
str_substr1a   movf     POSTINC0,w               ; avanca 1 byte na string
               movf     POSTINC1,w               ; avanca 1 byte na substring

               ; IF (S1 == DLM) { goto str_substr2; }
               movf     INDF0,w                  ; pega o caracter atual na string
               xorwf    str_dlm,w                ; verifica se e' delimitador
               bz       str_substr2              ; se for...
               ; IF (S2 == DLM) { goto str_substr2b; }
               movf     INDF1,w                  ; pega o caracter atual na substring
               xorwf    str_dlm,w                ; verifica se e' delimitador
               bz       str_substr2a             ; se for delimitador...
               ; IF (S1 == S2) { goto str_substr1a; }
               movf     INDF0,w                  ; pega o caracter atual na string
               xorwf    INDF1,w                  ; compara com o caracter atual na substring
               bz       str_substr1a             ; se iguais... 
               movf     str_reg4,w               ; restaura endereco Low da substring no stack
               movwf    FSR1L                    ; carrega indexador
               movf     str_reg3,w               ; restaura endereco High da substring no stack
               movf     FSR1H                    ; carrega indexador
               goto     str_substr0              ; procura primeiro caracter de onde parou na string
              ; IF (S2 == DLM) { Z=1; return} else {Z=0; return}
str_substr2    movf     INDF1,w                  ; pega  o caracter atual na substring
               xorwf    str_dlm,w                ; verifica se e' delimitador
               bz       str_substr2a             ; se for delimitador...
               bcf      STATUS,Z                 ; se nao for delimitador...
               return                            ; fim
str_substr2a
               movf     str_reg4,w               ; restaura endereco Low da substring no stack
               movwf    FSR1L                    ; carrega indexador
               movf     str_reg3,w               ; restaura endereco High da substring no stack
               movf     FSR1H                    ; carrega indexador
               movf     POSTDEC1,w               ; posiciona no tamanho da substring
               movf     INDF1,w                  ; pega o tamanho da substring
               subwf    FSR0L,w                  ; calcula LSB do endereco da substring na string
               movwf    str_reg2                 ; salva o LSB
               movf     STATUS,w                 ; pega o STATUS
               andlw    0x01                     ; deixa somente o Carry Bit
               xorlw    0x01
               subwf    FSR0H,w                  ; ajusta o MSB do endereco da substring na string
               movwf    str_reg1                 ; salva o MSB
               movf     str_reg6,w               ; pega o LSB do inicio da string
               subwf    str_reg2,f               ; calcula o LSB da posicao relativa da substring
               movf     STATUS,w                 ; pega o STATUS
               andlw    0x01                     ; deixa somente o Carry Bit
               xorlw    0x01
               subwf    str_reg1,f               ; ajusta o MSB da posicao relativa da substring
                                                 ; neste ponto  [str_reg1:str_reg2] tem a posicao
               movf     str_reg2,w               ; carrega a posicao relativa da substring 
               movwf    str_ptr                  ; carrega o registrador de posicao relativa
               movf     POSTDEC0, w              ; faz FSR0 apontar para o ultimo caracter da
                                                 ; substring dentro da string (23/04/2017)
               bsf      STATUS,Z                 ; achou a substring
               return                            ; fim 

; -----------------------------------------------------------------------------------------------
; Compara duas strings residentes na memoria RAM.                                      20/11/2014
;
; Entrada: FSR1 aponta str1
;          FSR2 aponta str2
;
; Saida  : STATUS<Z> = 1 --> str1 == str2
;          STATUS<Z> = 0 --> str1 != str2
;
; Funcao interna: nao deve ser chamada externamente.
;
; Se o tamanho das duas string forem iguais, faz a comparacao entre os caracteres 
; das duas strings senao e termina a comparacao indicando o resultado no flag Z
; do registrador STATUS.
;
; Se os tamanhos das strings forem difirentes, termina a comparacao com STATUS<Z>=0.
;
; -----------------------------------------------------------------------------------------------
str_cp         movf    POSTINC2,w                ; tamanho da str 2
               subwf   INDF1,w                   ; tamanho da str 1
               bnz     _str_f1                   ; se != 0 --> tamanhos diferentes
               movf    POSTINC1,w                ; pula o tamanho - aponta caracter 1
_str_cploop    movf    str_dlm,w                 ; pega o delimitador
               xorwf   INDF1,w                   ; se == 0 --> pega str 1
               bz      _str_f1                   ; se 0x00 --> fim
               movf    POSTINC1,w
               xorwf   POSTINC2,w                ; compara caracteres  str1 X str 2
               bz      _str_cploop               ; se iguais --> proximo
_str_f1        return                            ; fim


; -----------------------------------------------------------------------------------------------
; Formata, na RAM, a string 2 que esta inline na macro.                                20/11/2014
;
; Funcao interna: nao deve ser chamada externamente.
;
; Entrada: FSR1 aponta para o primeiro caracter no buffer auxiliar
; -----------------------------------------------------------------------------------------------
_str_fmt       movff   TOSU,TBLPTRU              ; topo do stack para UPPER
               movff   TOSH,TBLPTRH              ; topo do stack para HIGH
               movff   TOSL,TBLPTRL              ; topo do stack para LOW

               lfsr    FSR2, str_buffer           ; tamanho da str2 no buff auxiliar
               clrf    INDF2                     ; zera o tamanho da str 2
               
_str_fmt1      tblrd   *+                        ; le um caracter na str 2
               movf    TABLAT,w                  ; pega o caractrer lido
               movwf   POSTINC1                  ; move para o buffer auxiliar
               xorlw   0x00                      ; verifica o caracter lido
               bz      _str_fmtd                 ; se zero --> terminou 
               infsnz  INDF2,f                   ; soma 1 no tamanho da str 2
               goto    _str_e001                 ; se estourou --> erro
               goto    _str_fmt1                 ; continua formatando

_str_fmtd      movf    POSTDEC1,w
               movf    str_dlm,w
               movwf   POSTINC1
               btfsc   TBLPTRL,0                 ; endereco par???
               tblrd   *+                        ; NAO --> avanca para endereco par
               movf    TBLPTRU,w                 ; restaura o topo do stack
               movwf   TOSU                      ; restaura o topo do stack
               movf    TBLPTRH,w                 ; restaura o topo do stack
               movwf   TOSH                      ; restaura o topo do stack
               movf    TBLPTRL,w                 ; restaura o topo do stack
               movwf   TOSL                      ; restaura o topo do stack
               bsf     STATUS,Z                  ; STATUS<Z> = 0
               return                            ; fim

_str_e001      btfsc   TBLPTRL,0                 ; endereco par???
               tblrd   *+                        ; NAO --> avanca para endereco par
               movf    TBLPTRU,w                 ; restaura o topo do stack
               movwf   TOSU                      ; restaura o topo do stack
               movf    TBLPTRH,w                 ; restaura o topo do stack
               movwf   TOSH                      ; restaura o topo do stack
               movf    TBLPTRL,w                 ; restaura o topo do stack
               movwf   TOSL                      ; restaura o topo do stack
               bcf     STATUS,Z
               return

       
; -----------------------------------------------------------------------------------------------
; str_setdml - Especifica o delimitador para as funcoes de string                      20/11/2014
;
; Entrada: W contem o delimitador.
;
; Saida  : str_delimit carregado com o caracter delimitador
; -----------------------------------------------------------------------------------------------
str_setdlm     movwf   str_dlm
               return

       

; -----------------------------------------------------------------------------------------------
; str_schar - Procura um caracter na string.                                           20/11/2014
;
; Entrada: FSR0 aponta para o endereco do buffer a pesquisar.
;          W contem o caracter a procurar.
;
; Saida  : STATUS<Z>=0 --> caracter nao encontrado.
;          STATUS<Z>=1 --> W contem a posicao relativa do caracter encontrado.
;                          INDF0 aponta para o caracter encontrado
;
; Limite : maximo 128 caracteres. 
; -----------------------------------------------------------------------------------------------
str_schar      movwf   str_reg1                  ; salva o caracter a pesquisar
               clrf    str_ptr                   ; zera o apontador relativo
               movlw   .128                      ; limite maximo
               movwf   str_reg2                  ; carrega contador

str_schar1     movf    str_reg1,w                ; pega o caracter a pesquisar
               xorwf   INDF0,w                   ; pesquisa
               bz      str_schar3                ; se encontrou o caracter, sai do loop

               decfsz  str_reg2,f                ; estourou o limite???
               goto    str_schar2                ; NAO --> ainda nao
               goto    str_schar4                ; SIM --> estouro o limite

str_schar2     movf    str_dlm,w                 ; pega o delimitador
               xorwf   INDF0,w                   ; verifica
               bz      str_schar4                ; se achou delimitador --> fim

               movf    POSTINC0,w                ; aponta o proximo caracter na string
               incf    str_ptr,f                 ; senao, incrementa posicao relativa
               goto    str_schar1                ; pesquisa o proximo

str_schar3     movf    str_ptr,w                 ; carrega posicao relativa em W
               bsf     STATUS,Z                  ; fim normal
               return

str_schar4     movf    str_ptr,w                 ; carrega posicao relativa em W
               bcf     STATUS,Z                  ; fim sem encontrar o caracter
               return

; -----------------------------------------------------------------------------------------------
; srt_define - Define uma string na memoria RAM.                                       20/11/2014
;
; Entrada: FSR0 aponta para o byte +0 da string que contera o tamanho.
;          FSR1 aponta para o byte +1 da string onde inicia os caracteres.
;          O registrador 'srt_dml' deve conter o caracter delimitador de fim da string.
;
; Saida  : String definida no endereco requerido.
;          STATUS<Z>=1 ........ string criada com sucesso
;          STATUS<Z>=0 ........ string excedeu o tamanho limite
;
;  A string 'cccc......cc',  na memoria RAM (_addr_),sera configurada
;  com o formato 'LLcccc......ccDL', onde:
;  LL ..... e' o tamanho da string 
;  DL .....   o carater delimitador definido previamente pela 
;           funcao 'str_setdlm'.
;
;  Exemplo:    lfsr    FSR0, addr                ; endereco do tamanho na RAM
;              lfsr    FSR1, (addr + 1)          ; endereco dos caracteres na RAM
;              call    str_define                ; chama a funcao de definicao de string
;              DW      "Sistema ligado",0x00     ; string a ser definida                  
;
; -----------------------------------------------------------------------------------------------
str_define     movff   TOSU,TBLPTRU              ; topo do stack para UPPER
               movff   TOSH,TBLPTRH              ; topo do stack para HIGH
               movff   TOSL,TBLPTRL              ; topo do stack para LOW
               clrf    INDF0                     ; zera o tamanhos da string

_str_def1      tblrd   *+                        ; le um caracter na str 2
               movf    TABLAT,w                  ; pega o caractrer lido
               movwf   POSTINC1                  ; move para o buffer auxiliar
               xorlw   0x00                      ; verifica o caracter lido
               bz      _str_defd                 ; se zero --> terminou 
               infsnz  INDF0,f                   ; soma 1 no tamanho da str 2
               goto    _str_defe1                ; se estourou --> erro
               goto    _str_def1                 ; continua formatando

_str_defd      movf    POSTDEC1,w                ; retrocede e posiciona no delimitador
               movf    str_dlm,w                 ; pega o delimitador
               movwf   INDF1                     ; insere delimitador na string
               btfsc   TBLPTRL,0                 ; endereco par???
               tblrd   *+                        ; NAO --> avanca para endereco par
               movf    TBLPTRU,w                 ; restaura o topo do stack
               movwf   TOSU                      ; restaura o topo do stack
               movf    TBLPTRH,w                 ; restaura o topo do stack
               movwf   TOSH                      ; restaura o topo do stack
               movf    TBLPTRL,w                 ; restaura o topo do stack
               movwf   TOSL                      ; restaura o topo do stack
               bsf     STATUS,Z                  ; STATUS<Z> = 0
               return                            ; fim
_str_defe1     bcf     STATUS,Z                  ; erro
               return










; ---------------------------------------------------------------------------------------------
; mov_str - move uma string terminada por nulo (0x00) para o buffer                  27/11/2014
; apontado por FSR0.
;
; A string deve ser definida por 'DW' logo apos a chamada da funcao 'mov_str'
;
; Exemplo:     call    mov_str
;              DW      "Ola!!!"
;
; ---------------------------------------------------------------------------------------------
mov_str        movff   TOSU,TBLPTRU              ; topo do stack para UPPER
               movff   TOSH,TBLPTRH              ; topo do stack para HIGH
               movff   TOSL,TBLPTRL              ; topo do stack para LOW
               rcall   _mov_str_c                ; vai imprimir
               btfsc   TBLPTRL,0                 ; endereco par???
               tblrd   *+                        ; NAO --> avanca para endereco par
               movf    TBLPTRU,w                 ; restaura o topo do stack
               movwf   TOSU                      ; restaura o topo do stack
               movf    TBLPTRH,w                 ; restaura o topo do stack
               movwf   TOSH                      ; restaura o topo do stack
               movf    TBLPTRL,w                 ; restaura o topo do stack
               movwf   TOSL                      ; restaura o topo do stack
               return                            ; fim
; ---------------------------------------------------------------------------------------------
_mov_str_c
_movc         tblrd   *+                         ; le um caracter na lista
	          movf    TABLAT,w                   ; pega o caracter lido
	          bz      _fimmvstr                  ; termina se for nulo
              movwf     POSTINC0                 ; senao, imprime o caracter
              bra     _movc                      ; continua ate encontrar um nulo
_fimmvstr     return                             ; fim





; ---------------------------------------------------------------------------------------------
; cvstring                                                                           23/04/2015
;
; Copia uma string colocando o tamanho e o delimitador no destino.
;
; A string de origem deve ser delimitada com o delimitador especificado em 'srt_dlm".
;
;  String origem  -->  xxxxxxxxxxxxxxD
;  String destino -->  LxxxxxxxxxxxxxxD
;
;  Entrada: FSR0 aponta a string origem  -->  xxxxxxxxxxxxxxD
;           FSR1 aponta a string destino  
;
;  Saida  : A string destino estara no formato LxxxxxxxxxxxxxxD
;
; A localizacao de memoria de origem, apontada por FSR0, sera copiada para a localizaca destino,
; apontada por FSR1, ate que um delimitador, definido no registrador 'str_dlm', seja encontrado
; na origem. O primeiro byte do destino contera o tamanho e o ultimo byte o delimitador.
; Sendo assim, o destino devera ter 1 byte a mais do que a origem.
;
; ---------------------------------------------------------------------------------------------
cvstring     clrf    str_reg1          ; contador
             movf    FSR1H,w           ; destino
             movwf   FSR2H             ; salva endereco inicial
             movf    FSR1L,w           ; destino
             movwf   FSR2L             ; salva endereco inicial
             movf    POSTINC1,w        ; aponta para o primeiro 'x' no destino

_cvstrnxt    movf    INDF0,w           ; pega um 'x' na origem
             movwf   POSTINC1          ; move para o destino
             incf    str_reg1,f        ; incrementa o tamanho
             xorwf   str_dlm,w         ; verifica delimitador
             btfsc   STATUS,Z          ; fim da string origem???
             goto    _cvstrend         ; SIM --> fim
             movf    POSTINC0,w        ; aponta o proximo na origem
             goto    _cvstrnxt         ; proximo...

_cvstrend    movf    FSR2H,w
             movwf   FSR1H
             movf    FSR2L,w
             movwf   FSR1L
             decf    str_reg1,f        ; ajusta o tamanho
             movf    str_reg1,w        ; pega o tamanho
             movwf   INDF1             ; insere tamanho
             return



              ELSE

                ERROR "String-Services: Processador nao suportado"

              ENDIF              ; IF _SS_PROCESSOR_ == 18



; ---------------------------------------------------------------------------------------------
; _simple_mvc                                                                        21/05/2015
;
; Move localizacao da RAM de 'origem' com tamanho 'len' para a localizacao
; da RAM 'destino'.
;
; Esta funcao e' chamada pela macro MVC.
;
; Entrada: FSR0 aponta para a localizacao 'origem'
;          FSR1 aponta para a localizacao 'destino'
;          W contem o tamanho
;
; Saida  : Localizacao de 'destino' recebera uma copia da 'origem'.
;
; ---------------------------------------------------------------------------------------------
_simple_mvc
           movff   POSTINC0, POSTINC1
           decfsz  str_reg1,f
           goto    _simple_mvc
           return




; -------------------------------------------------------------------------------
; Extrair pares de numeros entre parenteses                            28/11/2015
;
; Este funcao so sera gerada se o simbolo  _EXTRAC_REQUIRED for definido
; no programa.
; Extrai um par de numeros que se encontram entre parentese dentro de uma string,
; cujos valores devem estar entre 0 e 255.
;
; Entrada: FSR0 aponta para a string no formato "LLsssssss...(XXX,YY)...ssss0D"
;
; Saida  : R1 contera o valor de XXX em decimal
;          R0 contera o valor de YY em decimal
;           W = 0x00 - sem erro
;           W = 0x01 - erro: nao encontrou "("
;           W = 0x02 - erro: nao encontrou "," separando os numeros
; -------------------------------------------------------------------------------
  ifdef _EXTRAC_REQUIRED
extract_int  
           call    clrdig
           lfsr    FSR2, DIGIT1

           movlw   0x0D
           xorwf   INDF0, w
           btfsc   STATUS, Z
           retlw   1                     ; erro: nao encontrou "("
           movlw   "("
           xorwf   POSTINC0, w
           btfss   STATUS, Z
           goto    extract_int

           movff   POSTINC0, POSTINC2    ; salva primeiro X
           movff   POSTINC0, POSTINC2    ; salva segundo  X
           movff   POSTINC0, POSTINC2    ; salva terceiro X

           pushR   FSR0H                 ; salva ponto onde parou
           pushR   FSR0L                 ; salva ponto onde parou
           pushR   FSR2H                 ; salva ponto onde parou
           pushR   FSR2L                 ; salva ponto onde parou

           movlw   0x0F
           andwf   DIGIT1, f
           andwf   DIGIT2, f
           andwf   DIGIT3, f
           clrf    DSIGN
           movlw   .3                    ; numero de digitos a converter
           call    dec2bin               ; converte a binario

           popR    FSR2L                 ; restaura ponto onde parou
           popR    FSR2H                 ; restaura ponto onde parou
           popR    FSR0L                 ; restaura ponto onde parou
           popR    FSR0H                 ; restaura ponto onde parou

           pushR   REGA0                 ; salva XXX convertido no stack

           call    clrdig
           lfsr    FSR2, DIGIT1
_extract01 movlw   0x0D
           xorwf   INDF0, w
           btfsc   STATUS, Z             ; fim da string?
           goto    _extrac_erro1         ; SIM --> sai com erro

           movlw   ","                   ; procura lela virgula
           xorwf   POSTINC0, w
           btfss   STATUS, Z             ; achou ","
           goto    _extract01   

           movff   POSTINC0, POSTINC2    ; salva primeiro Y
           movff   POSTINC0, POSTINC2    ; salva segundo  Y

           movlw   0x0F
           andwf   DIGIT1, f
           andwf   DIGIT2, f
           movlw   .2                    ; numero de digitos a converter
           call    dec2bin               ; converte a binario

           movff   REGA0, R0             ; R0 = YY
           popR    R1                    ; R1 = XXX
           retlw   .0                    ; retorna sem erro

_extrac_erro1
           popR    REGA0                 ; acerta o nivel do stack
           retlw   2                     ; erro: nao encontrou ","
   endif  ; ifdef _EXTRAC_REQUIRED
 endif  ;__STRING_SERVICE

