 ifndef __TERMINAL_SERVICE
#define  __TERMINAL_SERVICE 1   ; para outros servicos reconhecerem a
                                ; presenca deste servico
#define  __TERMINAL_SERVICES 1

; ---------------------------------------------------------------------------------------------
; Terminal  Services                                                               - 18/11/2014
;
; Servicos para operacao de Terminais (Hyperterminal do Windows e outros) atraves do
; modulo USART do microcontrolador PIC.
;
; REQUISITOS
; 1 - O usuario deve possuir o hardware necessario para fazer a interface entre o PIC e o PC.
; 2 - Esse software suporta somente microcontroladores da familia 18F.
; 3 - Este arquivo deve ser incluido no inicio do programa fonte.
; 4 - Definir no programa um buffer para leitura do teclado:
;
;     ser_buff   equ    0x200   ; qualquer lugar da RAM
;
;     TERM_READ  ser_buff, opt  ; leitura do teclado --> ver macro TERM_READ
;
; ---------------------------------------------------------------------------------------------
; Este software fornece uma serie de macro instrucoes que podem ser usadas para trabalhar
; com o terminal:
;
; TERM_CONFIG ............ Configura a USART nos modos sincrono ou assincrono.
;                          Esta macro deve ser emitida uma unica vez antes de iniciar
;                          as operacoes com o terminal.
;                          Exemplo:
;
;                          TERM_CONFIG .9600,.16000000,"A","N","C"  ; configura a USART
;
;
; TERM_NLSTR ............. Posiciona na linha seguinte do terminal e envia a string 
;                          inline na macro.
;                          Exemplo:  
;
;                          TERM_NLSTR "Isto e um teste"             ; envia mensagem
;
; TERM_STR ............... Envia a string inline na macro para o terminal. 
;                          Exemplo:

;                          TERM_STR "Isto e um teste, nl            ; envia mensagem
;
;
; TERM_GET ............... Le a porta RX da USART e retorna o caracter lido no registrador W.
;                          Exemplo:
;
;                          TERM_GET
;                          movwf    dado_lido
;
; TERM_PUT ............... Transmite o caracter do registrador W para o receptor via porta
;                          serial.
;                          Exemplo:
;
;                          movlw    "X"
;                          TERM_PUT
;
; TERM_NL ................ Posiciona na proxima lina do terminal.
;                          Exemplo:
;
;                          TERM_STR "Mensagem 1"
;                          TERM_NL
;                          TERM_STR "Mensagem 2"
;                          TERM_NL
;
;
; TERM_CLEAR ............. Apaga a tela do terminal.
;                          Exemplo:
;
;                          TERM_CLEAR
;
; TERM_MSG ............... Define uma mensagem na memoria de programa para ser enviada 
;                          posteriormente pela macro TERM_SENDMSG.
;                          Exemplo:
;
;                          TERM_MSG  msg001, "Mensagem 1",[nl/null]  
;
;
; TERM_SENDMSG ........... Envia, para o terminal,  uma mensagem criada pela macro TERM_MSG.
;                          Exemplo:
;
;                          TERM_SENDMSG  msg001, null
;
; TERM_READ .............. Le o terminal ate que um 0x0D (Enter) seja detectado.
;                          Exemplo:    
;
;                                    TERM_STR "Entre uma linha: "
;                                    TERM_READ SER_BUFF, SL               ; SL - se trabalhando com strings
;                                    TERM_NL
;                                    lfsr      FSR1,SER_BUFF              ; aponta o inicio do buffer
;                          print     movf      INDF1,w                    ; para um caracter do buffer
;                                    xorlw     0x0D                       ; verifica se acabou
;                                    btfsc     STATUS,Z                   ; acabou???
;                                    goto      segue                      ; SIM --> fim
;                                    movf      POSTINC1,w
;                                    TERM_PUT                             ; NAO --> imprime o caracter
;                                    goto      print                      ; vai pegar o proximo
;                            segue
;                                    TERM_NL                              ; vai pegar o proximo
; 
; ---------------------------------------------------------------------------------------------
   ifndef _termlab
          variable _termlab
_termlab     set      0
   endif
_termlab        SET       _termlab + 1

#define  null  .0       ; null
#define  SL    .1       ; para montar o buffer com tamanho e delimitado [Lxxxxxxx...xxD]
#define  lf    .10      ; line feed
#define  LF    .10      ; line feed
#define  cr    .13      ; carriage return
#define  CR    .13      ; carriage return
#define  nl    0x0D0A   ; new line

#define  txon     .1    ; TERM_SET txon ........ habilita transmissao
#define  txoff    .2    ; TERM_SET txoff ....... desabilita transmissao
#define  rxon     .3    ; TERM_SET rxon ........ habilita recepcao
#define  rxoff    .4    ; TERM_SET rxoff ....... desabilita recepcao
#define  usarton  .5    ; TERM_SET usarton ..... habilita a porta serial
#define  usartoff .6    ; TERM_SET usartoff .... desabilita a porta serial

; ---------------------------------------------------------------------------------------------
; MACRO: TERM_SET (Adaptacao da macro USART_SET de 15/02/2011)                       12/11/2015
;
; Modifica parametros da USART conforme valor do parametro.
;
;PROTOTIPO:
; USART_SET  rxtxmode
;
;       rxtxmode ............................ txon     - habilita transmissao
;                                             txoff    - desabilita a transmissao
;                                             rxon     - habilita a recepcao
;                                             rxoff    - desabilita a recepcao
;                                             usartoff - desabilita a porta serial
;
; EXEMPLO:
; USART_SET  txoff 
;
; ---------------------------------------------------------------------------------------------
TERM_SET      MACRO    rxtxmode

              if rxtxmode == txon
                   bsf    TXSTA,TXEN            ; habilita a transmissao
              endif

              if rxtxmode == txoff
                   bcf    TXSTA,TXEN            ; desabilita a transmissao
              endif

              if rxtxmode == rxon
                   bsf    RCSTA,CREN            ; habilita a recepcao
                   movf   RCREG,w               ;
                   movf   RCREG,w               ; limpa o buffer de leitura
                   movf   RCREG,w               ;      
              endif

              if rxtxmode == rxoff
                   bcf    RCSTA,CREN            ; desabilita a recepcao
              endif

              if rxtxmode == usarton
                 bsf    RCSTA,SPEN              ; habilita porta serial       
              endif

              if rxtxmode == usartoff
                 bcf    RCSTA,SPEN              ; desabilita porta serial       
              endif

              ENDM


; ---------------------------------------------------------------------------------------------
; TERM_SPACE - imprime um espaco no terminal
; ---------------------------------------------------------------------------------------------
TERM_SPACE     MACRO
               movlw  " "
               TERM_PUT
               ENDM


; ---------------------------------------------------------------------------------------------
; TERM_READ - Le o terminal ate que um 0x0D (Enter) seja detectado.
;
;
;
;      TERM_READ  buffer, opt
;
; buffer ......... deve ser um endereco na memoria RAM (dados) com capacidade
;                  suficiente para conter os dados esperados. O registrador FSR0 
;                  sera usado para enderecar o buffer. Nao ha controle de estouro
;                  do buffer.
;
; opt ............ SL -----> para trabalhar com funcoes de tratamento de strings fornecidas
;                            pelo servico "string_services".
;                  null ---> sem tratamento de strings.
;
; Exemplo:    
;           TERM_STR "Entre uma linha: "
;           TERM_READ SER_BUFF, null
;           TERM_NL
;           lfsr      FSR1,SER_BUFF              ; aponta o inicio do buffer
;print      movf      INDF1,w                    ; para um caracter do buffer
;           xorlw     0x0D                       ; verifica se acabou
;           btfsc     STATUS,Z                   ; acabou???
;           goto      segue                      ; SIM --> fim
;           movf      POSTINC1,w
;           TERM_PUT                             ; NAO --> imprime o caracter
;           goto      print                      ; vai pegar o proximo
;segue
;           TERM_NL
; ---------------------------------------------------------------------------------------------
TERM_READ  MACRO   buffer, opt
_termlab      set     _termlab + 1


  if opt==SL

  ifndef __dlmsw
    error "Delimitador de strings nao definido pela macro DLMSET"
  endif

  ; Quando se trabalha com as funcoes de tratamento de strings do arquivo
  ; "string.services.inc", este modo insere o tamanho da string no inicio do buffer
  ; e o delimitador, definido pela macro DLMSET, no fim do buffer.
           lfsr    FSR1,buffer                ; aponta para o tamanho no buffer
           clrf    INDF1                      ; limpa contador
           lfsr    FSR0,buffer+.1             ; aponta inicio dos caracteres no buffer 
_tread#v(_termlab)
           btfss   PIR1,RCIF                    
           goto    _tread#v(_termlab)
           movf    RCREG,w
           movwf   POSTINC0
           incf    INDF1,f                    ; incrementa o contador
           xorwf   str_dlm,w                  ; verifica delimitador
           btfss   STATUS,Z
           goto    _tread#v(_termlab)  
           decf    INDF1,f                 ; ajusta o contador

  else
  ; Este e' o modo normal que nao insere o tamanho no inicio do buffer
  ; mas somente o delimitador 0x0D definido localmente.
           lfsr    FSR0,buffer                ; aponta inicio do buffer  de leitura
_tread#v(_termlab)
           btfss   PIR1,RCIF                    
           goto    _tread#v(_termlab)
           movf    RCREG,w
           movwf   POSTINC0
           xorlw   0x0D
           btfss   STATUS,Z
           goto    _tread#v(_termlab)
   endif





           ENDM   



; ---------------------------------------------------------------------------------------------
; TERM_MSG - Define uma mensagem na memoria de programa para ser                   - 18/11/2014
;            enviada posteriormente pela macro TERM_SENDMSG.
;
; Exemplo:   TERM_MSG  msg001, "Mensagem 1", null
;
; ---------------------------------------------------------------------------------------------
TERM_MSG  MACRO msgname, msg, cntl
msgname   call  send_str
          DW    msg,0x00
      if cntl == nl
          TERM_NL
      endif
          return
          ENDM

; ---------------------------------------------------------------------------------------------
; TERM_SENDMSG - Envia, para o terminal, uma mensagem criada pela                  - 18/11/2014
;                macro TERM_MSG.
;                Se o parametro 'cntl' for 'null' nao muda de linha.
;                Se o parametro 'cntl' for 'nl' muda de linha apos a mensagem.
;
; Exemplo:     TERM_SENDMSG  msg001, nl
;
; ---------------------------------------------------------------------------------------------
TERM_SENDMSG   MACRO   msgname, cntl
               call    msgname
             if cntl == nl
               TERM_NL
             endif
               ENDM

; ---------------------------------------------------------------------------------------------
; TERM_NLSTR - Posiciona na linha seguinte do terminal (PC)                        - 18/11/2014
;              e envia a string inline na macro.
;
; Esta macro usa a funcao "..\..\SUBROUTINES.sys1\send_str.inc"
;
; Exemplo:     SEND_NLSTR  "Isto e uma mensagem"
; 
; ---------------------------------------------------------------------------------------------
TERM_NLSTR     MACRO   st
               call    send_str
               dw      0x0D0A,st,0x00
               ENDM
; ---------------------------------------------------------------------------------------------
; TERM_STR - Envia a string inline na macro para o terminal (PC)                   - 18/11/2014
;            Se o parametro 'cntl' for 'null' nao muda de linha.
;            Se o parametro 'cntl' for 'nl' muda de linha apos a mensagem.
;
; Esta macro usa a funcao "..\..\SUBROUTINES.sys1\send_str.inc"
;
; Exemplo:     SEND_STR  "Isto e uma mensagem", null
; 
; ---------------------------------------------------------------------------------------------
TERM_STR       MACRO   st, cntl
               call    send_str
               dw      st,0x00
             if cntl == nl
               TERM_NL
             endif
               ENDM


; ---------------------------------------------------------------------------------------------
; TERM_CONFIG - CONFIGURACAO DA USART                                              - 18/11/2014
;
;    Esta macro configura a USART nos modos sincrono ou assincrono considerando o valor do
;    baud rate desejado para a geracao do Baud Rate Generator (BRG). Permite tambem que se
;    faca a opcao por gerar a interrupcao de Rx.
;    Esta macro habilita a porta serial, o receiver (Rx) e o transmitter (tx).
;    A configuracao de I/O para Tx e Rx sera feita somente para a familia 18F.
;
;
;    TERM_CONFIG MACRO  bauds,clk, mode,interrupt
; 
;    bauds.......................... Baud Rate (BPS) desejado (4800, 9600, 19200, etc)
;    clk............................ frequenciao do clock (Fosc)
;    mode........................... ASYNH ou "A" - modo assincrono     
;                                    SYNCH ou "S" - modo sincrono
;    interrupt ..................... interrupcao de Rx
;                                    INTERRUPT ou "Y" - com interrupcao de Rx
;                                    NOINTERRUPT ou "N" - sem interrupcao de Rx
;    intertype ..................... LPRTY ou "L" - interrupcao ed Rx com baixa prioridade
;                                    HPRTY ou "H" - interrupcao ed Rx com alta prioridade
;                                    COMPAT ou "C" - interrupcao de Rx no modo normal (compativel)
;
; ---------------------------------------------------------------------------------------------
TERM_CONFIG  MACRO    bauds,clk,mode,interrupt,intertype
  ifndef term_usart_id
            variable term_usart_id
term_usart_id    set   1
SYNCH       equ   1                     ; modo assincrono
ASYNCH      equ   2                     ; modo sincrono
INTERRUPT   equ   3                     ; usa interrupcao
NOINTERRUPT equ   4                     ; nao usa interrupcao
HPRTY       equ   5                     ; interrupcao da alta prioridade 
LPRTY       equ   6                     ; interrupcao de baixa prioridade
COMPAT      equ   7                     ; interrupcao em modo de compatibilidade
;
txon        equ   1                     ; TERM_SET txon ........ habilita transmissao
txoff       equ   2                     ; TERM_SET txoff ....... desabilita transmissao
rxon        equ   3                     ; TERM_SET rxon ........ habilita recepcao
rxoff       equ   4                     ; TERM_SET rxoff ....... desabilita recepcao
usarton     equ   5                     ; TERM_SET usarton ..... habilita a porta serial
usartoff    equ   6                     ; TERM_SET usartoff .... desabilita a porta serial
  endif

         ; ----------------------------------------------------------------------------
         IF (mode == "A") && (bauds < .9600)
             bcf     TXSTA,SYNC                       ; modo assincrono
             bcf     BAUDCON,BRG16                    ; diz que SPBRG tem 8 bits
             bcf     TXSTA,BRGH                       ; estabelece baixa velocidade
             local  brg = clk / ( .64 * (bauds + .1))
         ENDIF

         IF (mode == ASYNCH) && (bauds < .9600)
             bcf     TXSTA,SYNC                       ; modo assincrono
             bcf     BAUDCON,BRG16                    ; diz que SPBRG tem 8 bits
             bcf     TXSTA,BRGH                       ; estabelece baixa velocidade
             local  brg = clk / ( .64 * (bauds + .1))
         ENDIF
         ; ----------------------------------------------------------------------------
         IF (mode == "A") && (bauds >= .9600) 
             bcf     TXSTA,SYNC                       ; modo assincrono
             bsf     BAUDCON,BRG16                    ; forca 16 bits em SPBRG
             bsf     TXSTA,BRGH                       ; estabelece alta velocidade
             local  brg = clk / ( .4 * (bauds + .1))
         ENDIF

         IF (mode == ASYNCH) && (bauds >= .9600) 
             bcf     TXSTA,SYNC                       ; modo assincrono
             bsf     BAUDCON,BRG16                    ; forca 16 bits em SPBRG
             bsf     TXSTA,BRGH                       ; estabelece alta velocidade
             local  brg = clk / ( .4 * (bauds + .1))
         ENDIF
         ; ----------------------------------------------------------------------------
         IF (mode == "S")
             bsf     TXSTA,SYNC                       ; modo sincrono
             bsf     BAUDCON,BRG16                    ; forca 16 bits em SPBRG
             local  brg = clk / ( .4 * (bauds + .1))
         ENDIF

         IF (mode == SYNCH)
             bsf     TXSTA,SYNC                       ; modo sincrono
             bsf     BAUDCON,BRG16                    ; forca 16 bits em SPBRG
             local  brg = clk / ( .4 * (bauds + .1))
         ENDIF
         ; ----------------------------------------------------------------------------

             movlw   0x80                             ; configuracao de I/O 
             iorwf   TRISC,F                          ;

             movlw   HIGH(brg)                        ; baud rate high

             movwf   SPBRGH                           ; estabelece o baud rate (high)
             movlw   LOW(brg)                         ; baud rate low

             movwf   SPBRG                            ; estabelece o baud rate (low)

	         bsf     RCSTA,SPEN                       ; habilita porta serial
             bsf     RCSTA,CREN                       ; habilita o receiver
             bsf     TXSTA,TXEN                       ; habilita o transmitter
         ; ----------------------------------------------------------------------------
         IF (interrupt == "Y") && (intertype == "H")
             bsf     PIE1,RCIE                        ; habilita interrupcoes em RX
             bsf     RCON,IPEN                        ; habilita prioridade de interrupcao
             bsf     IPR1,RCIP                        ; habilita interrupcao de alta prioridade em RX
             bsf     INTCON,GIEH                      ; habilita interrupcoes de alta prioridade
         ENDIF

         IF (interrupt == INTERRUPT) && (intertype == HPRTY)
             bsf     PIE1,RCIE                        ; habilita interrupcoes em RX
             bsf     RCON,IPEN                        ; habilita prioridade de interrupcao
             bsf     IPR1,RCIP                        ; habilita interrupcao de alta prioridade em RX
             bsf     INTCON,GIEH                      ; habilita interrupcoes de alta prioridade
         ENDIF
         ; ----------------------------------------------------------------------------
         IF (interrupt == "Y") && (intertype == "L")
             bsf     PIE1,RCIE                        ; habilita interrupcoes em RX
             bsf     RCON,IPEN                        ; habilita prioridade de interrupcao
             bcf     IPR1,RCIP                        ; habilita interrupcao de baixa prioridade em RX
             bsf     INTCON,GIEL                      ; habilita interrupcoes de baixa prioridade
         ENDIF

         IF (interrupt == INTERRUPT) && (intertype == LPRTY)
             bsf     PIE1,RCIE                        ; habilita interrupcoes em RX
             bsf     RCON,IPEN                        ; habilita prioridade de interrupcao
             bcf     IPR1,RCIP                        ; habilita interrupcao de baixa prioridade em RX
             bsf     INTCON,GIEL                      ; habilita interrupcoes de baixa prioridade
         ENDIF
         ; ----------------------------------------------------------------------------
         IF (interrupt == "Y") && (intertype == "C")
             bsf     PIE1,RCIE                        ; habilita interrupcoes em RX
             bsf     INTCON,PEIE                      ; habilita interrupcoes de perifericos
             bsf     INTCON,GIE                       ; habilita interrupcoes globalmente
         ENDIF

         IF (interrupt == INTERRUPT) && (intertype == COMPAT)
             bsf     PIE1,RCIE                        ; habilita interrupcoes em RX
             bsf     INTCON,PEIE                      ; habilita interrupcoes de perifericos
             bsf     INTCON,GIE                       ; habilita interrupcoes globalmente
         ENDIF
         ; ----------------------------------------------------------------------------
         ENDM


; ---------------------------------------------------------------------------------------------
; MACRO: TERM_GET                                                                  - 18/11/2014 
;
; Le a porta RX da USART e retorna o caracter lido no registrador W
;
; ---------------------------------------------------------------------------------------------
TERM_GET     MACRO
_termlab        set     _termlab + 1
_termRCV#v(_termlab)
             banksel PIR1
             btfss   PIR1,RCIF           ; verifica se ainda tem dado
             goto    _termRCV#v(_termlab)
             movf    RCREG,w             ; obtem o dado transmitido
  
             ENDM



; ---------------------------------------------------------------------------------------------
; MACRO: TERM_PUTCHAR                                                              - 02/04/2017
;
; Transmite o caracter inline para o receptor via porta serial.
; Suporta somente familia  18F.
;
; Antes de usar esta macro, a USART deve ser configurada com a macro
; TERM_CONFIG
;
; ---------------------------------------------------------------------------------------------
TERM_PUTCHAR  MACRO  char
_termlab      SET     _termlab + 1
             movlw  char                  ; pega o caracter
             movwf  TXREG                 ; transmite o caracter para o receptor
_termtx1#v(_termlab) btfss  TXSTA,TRMT            ; espera ate que a transmissao termine
             goto   _termtx1#v(_termlab)  ;
             ENDM


;
; ---------------------------------------------------------------------------------------------
; MACRO: TERM_PUT                                                                  - 18/11/2014
;
; Transmite o caracter do registrador W para o receptor via porta serial.
; Suporta somente familia  18F.
;
; Antes de usar esta macro, a USART deve ser configurada com a macro
; TERM_CONFIG
;
; ---------------------------------------------------------------------------------------------
TERM_PUT    MACRO
_termlab      SET     _termlab + 1
             movwf  TXREG                 ; transmite o caracter para o receptor
_termtx1#v(_termlab) btfss  TXSTA,TRMT            ; espera ate que a transmissao termine
             goto   _termtx1#v(_termlab)  ;
             ENDM


;
; ---------------------------------------------------------------------------------------------
; TERM_NL - Posiciona na proxima lina do terminal                                  - 18/11/2014
; ---------------------------------------------------------------------------------------------
TERM_NL    MACRO
        ;   call    send_str
        ;   DW      0x0D0A,0x00
           movlw 0x0D
           TERM_PUT
           movlw 0x0A
           TERM_PUT

           ENDM
;
; ---------------------------------------------------------------------------------------------
; TERM_CLEAR - Apaga a tela do terminal                                            - 18/11/2014
; ---------------------------------------------------------------------------------------------
TERM_CLEAR MACRO
           call _trm_clear
           ENDM


; ---------------------------------------------------------------------------------------------
; send_str - imprime, no terminal, uma string terminada por nulo (0x00).           - 18/11/2014
;
; A string deve ser definida por 'DW' logo apos a chamada da funcao 'send_str'
;
; Exemplo:     call    send_str
;              DW      "Ola!!!"
;
; ---------------------------------------------------------------------------------------------
send_str       movff   TOSU,TBLPTRU              ; topo do stack para UPPER
               movff   TOSH,TBLPTRH              ; topo do stack para HIGH
               movff   TOSL,TBLPTRL              ; topo do stack para LOW
              ; rcall   _send_str_c               ; vai imprimir
               call   _send_str_c               ; vai imprimir
               btfsc   TBLPTRL,0                 ; endereco par???
               tblrd   *+                        ; NAO --> avanca para endereco par
               movf    TBLPTRU,w                 ; restaura o topo do stack
               movwf   TOSU                      ; restaura o topo do stack
               movf    TBLPTRH,w                 ; restaura o topo do stack
               movwf   TOSH                      ; restaura o topo do stack
               movf    TBLPTRL,w                 ; restaura o topo do stack
               movwf   TOSL                      ; restaura o topo do stack
               return                            ; fim
; ---------------------------------------------------------------------------------------------
_send_char
_termlab       SET       _termlab + 1
               movwf     TXREG                     ; transmite o caracter para o receptor
_termtx2#v(_termlab)
               btfss     TXSTA,TRMT                ; espera ate que a transmissao termine
               goto      _termtx2#v(_termlab)      ;
               return
; ---------------------------------------------------------------------------------------------
; 
; ---------------------------------------------------------------------------------------------
_send_str_c
_sendc         tblrd   *+                        ; le um caracter na lista
	           movf    TABLAT,w                  ; pega o caracter lido
	           bz      _fimstr                   ; termina se for nulo
               call    _send_char                ; senao, imprime o caracter
               bra     _sendc                    ; continua ate encontrar um nulo
_fimstr        return                            ; fim


_trm_cls     DB      0x1B,"[","H",0x1B,"[","J",0x00
_trm_clear   movlw   UPPER(_trm_cls)
             movwf   TBLPTRU
             movlw   HIGH(_trm_cls)
             movwf   TBLPTRH
             movlw   LOW(_trm_cls)
             movwf   TBLPTRL
             call    _xmseq0
             return

_send_static
_xmseq0      tblrd   *+
             movf    TABLAT,w
             xorlw   0x00
             btfsc   STATUS,Z
             return
             movwf   TXREG
_xmseq1      btfss   TXSTA,TRMT
             goto    _xmseq1
             goto    _xmseq0

  endif  ;__TERMINAL_SERVICE

